/* globals $GC */
/**
 * 描述: 微医H5基础逻辑
 * 时间：2017年1月整理
 */
'use strict';
var $GH = (function($) {

    return {
        /**
         * 根据gp-xxxx选择器来框定$controller
         * $GH.module('.gp-hospital-homepage', function($controller){...}
         * @param  {[type]}   controller css selector
         * @param  {Function} fn
         * @return {[type]}
         */
        module: function(controller, fn) {
            var $controller = $(controller);
            if ($controller.length > 0) {
                var scope = window.alight.Scope();
                fn.call(this, $controller, scope);
                window.alight.applyBindings(scope, $controller[0]);
            }
        },
        /**
         * 获取GET请求
         * @param  {[type]} str [description]
         * @return {[type]}     [description]
         */
        getQuery: function(str) {
            if (typeof str === "string") {
                str = str.replace(/[^\?]+\?/, "?");
            }
            return (str || document.location.search).replace(/(^\?)/, '').split("&").map(function(n) {
                return n = n.split("="), this[n[0]] = n[1], this;
            }.bind({}))[0];
        },
        /**
         * 即将废弃！！！请使用$.cookies！！！
         */
        cookie: {
            set: function(cname, cvalue, exdays, path, domain) {
                var d = new Date();
                d.setTime(d.getTime() + (exdays * 24 * 60 * 60 * 1000));
                var expires = "expires=" + d.toGMTString();
                path = '; path=' + (path || '/');
                domain = domain ? '; domain=' + domain : '';
                document.cookie = cname + "=" + cvalue + "; " + expires + path + domain;
            },
            get: function(cname) {
                if (document.cookie.length > 0) {
                    var cStart = document.cookie.indexOf(cname + "=");
                    if (cStart !== -1) {
                        cStart = cStart + cname.length + 1;
                        var cEnd = document.cookie.indexOf(";", cStart);
                        if (cEnd === -1) {
                            cEnd = document.cookie.length;
                        }
                        return document.cookie.substring(cStart, cEnd);
                    }
                }
                return "";
            }
        },
        /**
         * 即将废弃，请使用$.formatDate
         */
        genDate: function(d) {
            return ((d.getMonth() + 1) < 10 ? '0' + (d.getMonth() + 1) : (d.getMonth() + 1)) +
                '-' + (d.getDate() < 10 ? '0' + d.getDate() : d.getDate()) +
                ' ' + (d.getHours() < 10 ? '0' + d.getHours() : d.getHours()) +
                ':' + (d.getMinutes() < 10 ? '0' + d.getMinutes() : d.getMinutes()) +
                ':' + (d.getSeconds() < 10 ? '0' + d.getSeconds() : d.getSeconds());
        },
        /**
         * 即将废弃，请使用$.formatDate
         */
        getTime: function(format, timestamp) {
            var t = (typeof timestamp === "number" || timestamp instanceof Date) ? new Date(timestamp) : new Date(),
                o = {
                    y: t.getFullYear(),
                    m: t.getMonth() + 1,
                    d: t.getDate(),
                    h: t.getHours(),
                    i: t.getMinutes(),
                    s: t.getSeconds(),
                };
            for (var a in o) {
                o[a.toUpperCase()] = (o[a] < 10 ? "0" : "") + o[a];
            }
            o.w = t.getDay();
            o.W = "日一二三四五六".substr(o.w, 1);
            o.timestamp = t.getTime();
            o.date = t;
            if (typeof format === "string") {
                for (var a in o) {
                    format = format.replace(new RegExp("(" + a + ")", "g"), function($0, $1) {
                        return o[$1];
                    });
                }
                return format;
            } else {
                return o;
            }
        },
        // Log 用户行为操作
        monitor: {
            fpCode: 0,
            /**
             * 浏览器指纹识别功能
             * @return {[type]} [description]
             */
            fingerPrint: function() {
                var cacheCode = $.cookies('_fp_code_');
                if ((!cacheCode || cacheCode.length !== 32) && window.Fingerprint2) {
                    new window.Fingerprint2().get(function(result) {
                        $.cookies('_fp_code_', result, {
                            path: '/',
                            expires: 365 * 10
                        });
                        $GH.monitor.fpCode = result;
                    });
                } else {
                    $GH.monitor.fpCode = cacheCode;
                }
            },
            /**
             * 跟踪来源 - 精准打点跟踪分析
             * @return {[type]} [description]
             */
            trackSource: function() {
                var source = $.getUrlParams('trackSource') || $.getUrlParams('_f');
                if (source) {
                    $.cookies('_track_source', source, {
                        path: '/',
                        expires: 1
                    });
                }
            }
        },
        /**
         * Angular Light 模板编译
         * @method  alTmpl
         * @example  <span al-if="data.colligateStatus == '2'" class="gbs gbt-third J_Resend" data-id="{{data.orderNo}}">补发短信</span>
         * @param   {string}template 模板字符串
         * @param   {object}data 数据
         * @return  {void}
         */
        alTmpl: function(template, data) {
            var tag = document.createElement('div'), // make an element
                scope = window.alight.Scope(); // make a scope
            tag.innerHTML = template; // set up template
            scope.data = data; // set init value
            window.alight.applyBindings(scope, tag); // apply bindings
            return tag.innerHTML;
        },
        /**
         * 字符串截取
         * @param  {[type]} str       [description]
         * @param  {[type]} maxLength [description]
         * @return {[type]}           [description]
         */
        smartStr: function(str, maxLength) {
            return str.length > maxLength ? str.substr(0, maxLength) + '...' : str;
        },
        /**
         * 字符串模糊化
         * @param  {[String]} str  输入字符串
         * @param  {[Number]}  from  开始位置
         * @param  {[Number]}  to    结束位置
         * @return {[String]}           模糊化结果
         */
        maskStr: function(str, from, to) {
            if (!to) {
                to = str.length;
            }
            return str.substring(0, from - 1) + str.substring(from - 1, to).replace(/./g, "*") + str.substring(to);
        },
        /**
         * 单位转换为万
         * @param  {[type]} count [description]
         * @return {[type]}       [description]
         */
        smartCount: function(count) {
            var smartCount = (count / 10000).toFixed(1);
            if (smartCount >= 1) {
                count = smartCount + '万';
            }
            return count;
        },
        /**
         * 直接使用原生的iscroll总有锁死问题
         * @param  {[type]} selector [description]
         * @param  {[type]} options  [description]
         * @return {[type]}          [description]
         */
        iScroll: function(selector, styles, options) {

            if ($('#gh').length === 0 && $(selector).parent().length > 0) {
                // 这一段是什么用处？
                var dom = $(selector).parent();
                if (dom.attr("id") === "J_MultiOptions_Div") {
                    var text = dom.find("h3").text();
                    if (!!text) {
                        styles.top = dom.find("h3").height();
                    }
                }
            }
            // 自动适应是否有头尾
            styles.top = (styles.top || 0) + $('#gh').height();
            styles.bottom = styles.bottom || 0;

            var wrapperStyle = $.extend({}, {
                position: 'absolute',
                left: 0,
                top: 0,
                bottom: 0,
                zIndex: 1,
                width: '100%',
                overflow: 'hidden',
                display: 'block'
            }, styles || {});

            options = $.extend({}, {
                scrollbars: true,
                fadeScrollbars: true,
                shrinkScrollbars: 'scale',
                mouseWheel: true,
                click: true,
                tap: "iscrollTap",
                disablePointer: true,
                disableTouch: false,
                disableMouse: false
            }, options);

            // 设置样式
            $(selector).css(wrapperStyle);

            if (window.IScroll) {
                return new window.IScroll(selector, options);
            }
        },
        /**
         * 通用遮罩层
         * @param {[type]} options [description]
         */
        Wrapper: function(options) {
            var wHeight = document.body.scrollHeight > window.screen.availHeight ?
                document.body.scrollHeight + 'px' : window.screen.availHeight + 'px';
            this.config = $.extend({}, {
                zIndex: 100,
                opacity: 0.5,
                width: '100%',
                height: wHeight,
                top: 0,
                background: '#000'
            }, options || {});

            this.$wrapper = $('#J_wrapper').length > 0 ?
                $('#J_wrapper') : (function() {
                    $('body').append($('<div id="J_wrapper"></div>'));
                    return $('#J_wrapper');
                })();

            $.extend($GH.Wrapper.prototype, {
                show: function(cb) {
                    var self = this;

                    $("body").css("overflow", "hidden");

                    // lock
                    $(document).on('ontouchstart', function(e) {
                        e.preventDefault();
                    }).on('touchmove', function(e) {
                        e.preventDefault();
                    });

                    self.$wrapper.css({
                        position: 'absolute',
                        opacity: 0,
                        background: self.config.background,
                        zIndex: self.config.zIndex,
                        top: self.config.top,
                        width: self.config.width,
                        height: self.config.height
                    }).animate({
                        opacity: self.config.opacity
                    }, 100, function() {
                        if ($.isFunction(cb)) {
                            cb.call(self);
                        }
                    });

                    return self.$wrapper;
                },
                close: function() {
                    this.$wrapper.remove();
                    // release
                    $(document).off('ontouchstart').off('touchmove');
                    $("body").css("overflow", "auto");
                }
            });
        },
        /**
         * 底部filter
         * @param  {[Object]} dom bottom filter节点
         * @param  {[Function]} fn 详细菜单上个性化函数定义
         * @param  {[Function]} navReset nav菜单打开时reset函数定义
         * @return {[type]} [description]
         */
        botFilter: function(dom, fn, navReset) {

            if (!dom.length) {
                return;
            }

            var _this = this;


            var mask = $(".g-mask").length ? $(".g-mask") : $('<div class="g-mask"></div>').appendTo("body").hide();

            dom.find(".J_Nav").css("max-height", $(window).height() - 50 + "px");

            function open() {
                var index = dom.find(".J_Menu").index(_this.menu);
                mask.show();
                dom.find(".J_Menu_Link").removeClass("on");
                dom.find(".J_Menu").removeClass("on").removeClass("open");
                dom.find(".J_Menu:eq(" + index + ")").addClass("on").addClass("open");
                dom.find(".J_Nav").hide();
                dom.find(".J_Nav:eq(" + index + ")").show();

                if (navReset) {
                    navReset.call(null, index);
                }
            }

            var close = _this.close = function() {
                mask.hide();
                dom.find(".J_Nav").hide();
                dom.find(".J_Menu").removeClass("open");
            };


            mask.on("click", function() {
                close();
                return false;
            });

            dom.on("click", ".J_Menu", function() {
                _this.menu = $(this);
                if (_this.menu.hasClass("open")) {
                    close();
                } else {
                    open();
                }
                return false;
            });

            fn.call(null, _this);

            return _this;

        },
        /**
         * 设置按钮状态
         * @type {Object}
         */
        btn: {
            disable: function(btn, text) {
                btn.data("disabled", true);
                if (btn.hasClass("gbn") || btn.hasClass("gbs")) {
                    btn.addClass("gbt-off");
                } else if (btn.hasClass("btn")) {
                    // 新版按钮样式
                    btn.addClass("disabled");
                }
                btn.text(text);
            },
            enable: function(btn, text) {
                btn.data("disabled", false);
                if (btn.hasClass("gbn") || btn.hasClass("gbs")) {
                    btn.removeClass("gbt-off");
                } else if (btn.hasClass("btn")) {
                    // 新版按钮样式
                    btn.removeClass("disabled");
                }
                btn.text(text);
            },
            isActive: function(btn) {
                return !btn.data("disabled");
            }
        },
        /**
         * 用户点击时候按钮的样式名称切换(GJ_TapClass)
         * 要定义切换的样式名在元素属性加入data-tap-class="????"
         * @author linyu
         * @return {undefined}
         */
        btnTapClassChange: function() {
            if ($('.GJ_TapClass').length === 0) {
                return;
            }
            $('.GJ_TapClass').on('touchstart', function() {
                var $this = $(this),
                    className = $this.data("tap-class") || "active";
                $this.addClass(className);
            });
            $('.GJ_TapClass').on('touchend', function() {
                var $this = $(this);
                $this.removeClass($this.data("tap-class") || "active");
            });
        },
        /**
         * 提示信息
         * @param  msg      消息
         * @param  duration 持续时间
         * @return
         */
        alert: function(msg, duration, callback) {
            if (!msg) {
                return;
            }
            $.pgwModal({
                content: msg,
                popup: true,
                // 用了CSS动画，动画时间2s，这个参数就没用了
                duration: duration || 2000,
                close: false,
                afterClose: function() {
                    if (typeof callback === 'function') {
                        callback();
                    }
                }
            });
        },
        /**
         * 确认框
         * @param  {[type]} msg            提示信息
         * @param  {[type]} okCallback     确定回调
         * @param  {[type]} cancelCallback 取消回调
         * @return {[type]}
         */
        confirm: function(msg, confirmButtons, customClass) {
            if (typeof confirmButtons === "function") {
                $.pgwModal({
                    content: '<div class="g-msg">' + msg + '</div>',
                    popup: true,
                    close: false,
                    confirm: true,
                    confirmButtons: [{
                        name: '确定',
                        style: 'btn',
                        fn: function(close) {
                            close();
                            confirmButtons();
                        }
                    }, {
                        name: '取消',
                        style: 'btn',
                        fn: function(close) {
                            close();
                        }
                    }]
                });
            } else {
                $.pgwModal({
                    content: '<div class="g-msg">' + msg + '</div>',
                    popup: true,
                    close: false,
                    confirm: true,
                    customClass: customClass,
                    confirmButtons: confirmButtons
                });
            }
        },
        /**
         * 通用Loading效果
         * @type {Object}
         */
        loading: {
            show: function() {
                if ($('.GJ_Loading').length > 0) {
                    if (!$('.GJ_Loading').is(':visible')) {
                        $('.GJ_Loading').show();
                    }
                    return;
                }
                $('body').append('<div class="g-page-loading GJ_Loading"><div class="roll"></div><div class="fixed"></div></div>');
            },
            hide: function() {
                if ($('.GJ_Loading').length === 0) {
                    return;
                }
                $('.GJ_Loading').hide();
            }
        },
        /**
         * xss攻击防护
         * @param  {[type]} code    [description]
         * @param  {[type]} options [description]
         * @return {[type]}         [description]
         */
        filterXSS: function(code, options) {
            if (window.filterXSS) {
                return window.filterXSS(code, options);
            } else {
                console.warn("filterXSS 未引入");
                return code;
            }
        },
        /**
         * 身份证验证
         * --
         * @author 楼云龙
         * @version 2015.06.15
         * @description 用于验证大陆15位或18位身份证的有效性，并获取相关信息
         * @param {boolean} backInfo true-返回身份证信息（生日、性别等），false-返回身份证验证结果
         */
        verifyIdcard: function(id, backInfo) {
            var info = {
                    y: "1900",
                    m: "01",
                    d: "01",
                    sex: "male",
                    valid: false,
                    length: 0
                },
                initDate = function(length) {
                    info.length = length;
                    var a = length === 15 ? 0 : 2, // 15:18
                        temp;
                    info.y = (a ? "" : "19") + id.substring(6, 8 + a);
                    info.m = id.substring(8 + a, 10 + a);
                    info.d = id.substring(10 + a, 12 + a);
                    info.sex = id.substring(14, 15 + a) % 2 === 0 ? "female" : "male";
                    temp = new Date(info.y, info.m - 1, info.d);
                    return (temp.getFullYear() == info.y * 1) && (temp.getMonth() + 1 == info.m * 1) && (temp.getDate() == info.d * 1);
                },
                back = function() {
                    return backInfo ? info : info.valid;
                };
            if (typeof id !== "string") {
                return back();
            }
            // 18
            if (/^\d{17}[0-9x]$/i.test(id)) {
                if (!initDate(18)) {
                    return back();
                }
                id = id.toLowerCase().split("");
                var wi = [7, 9, 10, 5, 8, 4, 2, 1, 6, 3, 7, 9, 10, 5, 8, 4, 2],
                    y = "10x98765432".split(""),
                    sum = 0;
                for (var i = 0; i < 17; i++) {
                    sum += wi[i] * id[i];
                }
                if (y[sum % 11] === id.pop().toLowerCase()) {
                    info.valid = true;
                }
                return back();
            }
            // 15
            else if (/^\d{15}$/.test(id)) {
                if (initDate(15)) {
                    info.valid = true;
                }
                return back();
            } else {
                return back();
            }
        },

        /**
         * 调整内容块padding top/bottom 适应顶部、底部fixed模块
         * @param  {Function} callback 成功、出错回调函数
         */
        adjustGpPadding: function(callback) {
            var gp = $(".GJ_Gp");

            // padding-bottom adjust
            var bottom = 10;
            if ($("#gf").length) {
                var gfHeight = $("#gf").height();
                $(".GJ_BotFixed:visible").css("bottom", gfHeight + "px");
                bottom += gfHeight;
            }
            gp.css("padding-bottom", bottom + $(".GJ_BotFixed:visible").height() + "px");

            // padding-top adjust
            var top = 0;
            if ($("#gh").length) {
                var ghHeight = $("#gh").height();
                $(".GJ_TopFixed:visible").css("top", ghHeight + "px");
                top += ghHeight;
            }
            if (gp.hasClass("gp-tp")) {
                top += 10;
            }
            gp.css("padding-top", top + ($(".GJ_TopFixed:visible").height() - 1) + "px");

            if (callback) {
                callback();
            }
        },

        /**
         * 表单异步提交 v2
         * @param  {[type]}   $form   表单zepto对象
         * @param  {Object} $trigger 提交触发节点
         * @param  {Boolean} $trigger 提交触发节点
         */
        formSubmitV2: function($form, $trigger) {

            if (!$trigger.loading) {
                var txt = "中...";
                $trigger.loading = true;
                $trigger.text($trigger.text() + txt);
                $.ajax({
                    dataType: "json",
                    cache: false,
                    data: $form.serialize(),
                    url: $form.attr("action"),
                    type: $form.attr("method"),
                    success: function(data) {
                        if (data.hasError === undefined || data.hasError) {
                            $GH.alert(data.message);
                            if ($(".GJ_ImgCaptcha").length > 0) {
                                $GH.refreshCaptcha($(".GJ_ImgCaptcha"));
                            }
                        } else {
                            if (data.returnUrl && data.returnUrl != "") {
                                var url = data.returnUrl;
                                if (url.indexOf("http://") < 0 && url.indexOf("https://") < 0) {
                                    url = $GC.guahaoServer + url;
                                }
                                location.href = url;
                            }
                        }
                    },
                    error: function(xhr, code) {
                        if (code === 'abort') {
                            return;
                        }
                        $GH.alert("系统异常，请稍后重试！");
                    },
                    complete: function(xhr, code) {
                        var str = $trigger.text();
                        $trigger.loading = false;
                        $trigger.text(str.substring(0, str.indexOf(txt)));
                    }
                });
            }

        },

        /**
         * 表单异步提交
         * @param  {[type]}   $form   表单zepto对象
         * @param  {Object} fns 成功、出错回调函数
         */
        formSubmit: function($form, fns) {
            $GH.loading.show();
            $.ajax({
                dataType: "json",
                cache: false,
                data: $form.serialize(),
                url: $form.attr("action"),
                type: $form.attr("method"),
                success: function(data) {
                    $GH.loading.hide();
                    // console.log(data);
                    if (data.hasError === undefined || data.hasError) {
                        if (fns && fns.error) {
                            fns.error.call($form, data);
                        } else {
                            $GH.alert(data.message);
                        }
                    } else {
                        if(fns && fns.confirm) {
                            fns.confirm.call($form, data);
                        } else {
	                        var msg = data.message || '信息提交成功';
	                        $GH.alert(msg, 2000, function() {
                                if (fns && fns.success) {
                                    fns.success.call($form, data);
                                } else {
                                    if (data.returnUrl && data.returnUrl != "") {
                                        var url = data.returnUrl;
                                        if (url.search(/^((http|https):\/\/)/) < 0) {
                                            url = $GC.guahaoServer + url;
                                        }
                                        location.href = url;
                                    }
                                }
                            });
                        }
                    }
                },
                error: function(xhr, code) {
                    $GH.loading.hide();
                    if (code === 'abort') {
                        return;
                    }
                    $GH.alert("系统异常，请稍后重试！");
                }
            });
        },

        /**
         * 预加载图片
         * --
         * @author 楼云龙
         * @param {array} urls 图片链接地址
         * @example
         *     // 图片加载完成后再处理
         *     new $GH.Preload("a.png").then(function () {
         *         // do...
         *     });
         *
         *     // 返回Preload对象，实现链式异步回调
         *     // 按序加载图片
         *     new $GH.Preload("a.png").then(function () {
         *         // do...
         *         return new $GH.Preload("b.png");
         *     }).then(function () {
         *         // do...
         *     });
         */
        Preload: function() {
            var P = function(urls) {
                var p = this;
                p.queue = [];
                p.urls = [].concat(urls);
                p.total = p.urls.length;
                p.count = 0;
                p.urls.forEach(function(url) {
                    if (typeof url !== "string") {
                        ++p.count;
                        return false;
                    }
                    var img = document.createElement("img");
                    img.src = url;
                    img.onload = img.onerror = function() {
                        if (++p.count >= p.total) {
                            var fn = p.queue.shift();
                            if (typeof fn === "function") {
                                var res = fn();
                                if (res && res.constructor === P) {
                                    res.queue = p.queue;
                                }
                            }
                        }
                    };
                });
            };
            P.prototype = {
                constructor: P,
                then: function(fn) {
                    this.queue.push(fn);
                    return this;
                }
            };
            return P;
        }(),

        /**
         * ajax封装
         * --
         * @author 楼云龙
         * @description ajax请求，内部封装了错误处理，屏蔽重复请求，用法同$.ajax
         * @example
         *     $G.request({
         *         url: "",
         *         success: function(res){
         *             // 这里无需判断hasError，即可直接使用res.data
         *         }
         *     });
         * 实现如下功能：
         *   ·内部集成了错误处理，并统一使用弹窗提示错误信息，
         *      h5的$GH.alert，portal的$GM.smartAlert，gops的$G.dialog.alert
         *   ·使用队列储存请求地址，如果请求地址已存在于队列中，则阻止请求
         */
        request: function() {
            var requestQueue = {};
            return function(options) {
                if (!options) {
                    options = {};
                }
                // 处理函数，长期使用起来，发现并没有必要使用别名，
                // 所以融合fulfilled与success，rejected与error，always与complete
                var successHandler = options.fulfilled || options.success;
                var errorHandler = options.rejected || options.error;
                var completeHandler = options.always || options.complete;
                var url = options.url || "";

                var ident = url + "|" + JSON.stringify(options.data);
                if (ident in requestQueue) {
                    return false;
                }
                requestQueue[ident] = true;

                var _options = {
                    url: url,
                    cache: false,
                    dataType: "json",
                    success: function(res) {
                        if (!res || res.hasError == 1) {
                            $GH.alert(res && res.message || "操作失败");
                            errorHandler && errorHandler.call(null, res, true);
                        } else {
                            // 成功由fulfilled定制响应方式，失败调用统一的dialog显示message
                            successHandler && successHandler.call(null, res);
                        }
                    },
                    error: function(err) {
                        if (window.console) {
                            console.log(err);
                        }
                        if (err.statusText && err.statusText !== "abort") { // 手动中断的不提示
                            $GH.alert("网络连接失败");
                        }
                        errorHandler && errorHandler.call(null, err);
                    },
                    complete: function() {
                        delete requestQueue[ident];
                        completeHandler && completeHandler.call(null);
                    }
                };

                delete options.success;
                delete options.error;
                delete options.complete;

                $.extend(_options, options);

                return $.ajax(_options);
            };
        }(),

        /**
         * 底部弹出报告，用于手机端调试，支持打印一些字符串信息
         * @param  {String} msg 要打印的字符串
         * @example
         *
         * $GH.report("Hello World!");
         * $GH.report("Hello World!", {top: true, height: "100%"});  // 自定义弹出层属性
         */
        report: function(msg) {
            var initialized = false;
            var elem;
            var report = function(msg, _options) {
                report.add(msg, _options);
                return report;
            };
            var init = function() {
                elem = document.createElement("div");
                elem.style.cssText = "z-index: 2000;position: fixed;bottom: 0;width: 100%;height: 4.8em;background-color: rgba(0,0,0,0.7);color: #fff;font-size: 1em;line-height: 1.2em;word-break: break-all;z-index: 1002;word-wrap: break-word;overflow-y: auto;";
                document.body.appendChild(elem);
                elem.addEventListener("click", report.clear);
                initialized = true;
            };

            report.add = function(msg, _options) {
                if (!_options) {
                    _options = {};
                }
                var options = {
                    top: _options.top || false, // 在顶部显示弹出层
                    height: _options.height || "4.8em", // 弹出层高度
                    sep: _options.sep || " ", // 分隔符
                };
                if (!initialized) {
                    init();
                }
                if (msg === null) {
                    msg = "null";
                } else if (msg === void 0) {
                    msg = "undefined";
                } else {
                    msg = msg.toString();
                    msg = msg.replace(/&/g, "&amp").replace(/</g, "&lt").replace(/>/g, "&gt");
                }
                elem.innerHTML += msg + options.sep;
                elem.style.display = "";
                elem.style[options["top"] ? "top" : "bottom"] = "0";
                elem.style[options["top"] ? "bottom" : "top"] = "";
                elem.style.height = options.height;
                return report;
            };

            report.clear = function() {
                if (!initialized) {
                    init();
                }
                elem.innerHTML = "";
                elem.style.display = "none";
                return report;
            };

            return report;
        }(),

        /**
         * 自适应高度
         * --
         * @author 楼云龙
         * @description 容器高度发生变化前执行该函数，配合transition实现缓动
         *              执行autoHeight -> 记录原高度 -> 操作DOM -> 获取新高度 -> 强制渲染 -> 触发动画
         * @param {dom} elem 原生dom对象
         * @param {string} transition 自定义css的transition
         * @example
         *     autoHeight(elem);
         *     $(elem).append("<div>123</div>"); // 可用于查看更多，tab切换等容器的高度过渡动画
         */
        autoHeight: function(elem, transition) {
            if (elem.length) {
                elem = elem[0];
            }
            if (!elem || elem.nodeType !== 1) {
                return false;
            }

            var oldHeight = elem.offsetHeight,
                newHeight;
            if (!transition) {
                transition = "height 0.24s ease-out";
            }
            elem.style.height = oldHeight + "px";

            setTimeout(function() {
                elem.style.transition = "none";
                elem.style.height = "auto";
                newHeight = elem.offsetHeight;
                elem.style.height = oldHeight + "px";

                elem.offsetHeight; // 强制渲染
                elem.style.transition = transition;
                elem.style.height = newHeight + "px";
            }, 0);
        },

        /**
         * 过渡动画
         * --
         * @author 楼云龙
         * @param {jquery} elem jQuery对象，需要操作的DOM元素
         * @param {object} options 参数
         *     duration 300,  // 动画持续时间
         *     active: "active",  // 激活元素的选择器
         *     transition: "transtion"  // 过渡动画的选择器
         *
         * @example
         *     css:
         *         .box{
         *             display: none;opacity: 0;transition: opacity .5s ease-out;
         *         }
         *         .box.active{  // 激活元素的选择器
         *             display: block;
         *         }
         *         .box.transition{  // 过渡动画选择器
         *             opacity: 1;
         *         }
         *     js:
         *         var boxViewer=new $GU.Viewer($("#box"), {
         *             duration: 500
         *         });
         *         boxViewer.show();
         *         boxViewer.hide();
         */
        Viewer: function() {
            function V(elem, options) {
                var _options = {
                    duration: 300,
                    active: "active",
                    transition: "transition"
                };
                this.options = $.extend({}, _options, options);
                this.timer = {
                    show: -1,
                    hide: -1
                };
                this.elem = $(elem);
            }
            V.prototype = {
                show: function(cb) {
                    clearTimeout(this.timer.show);
                    clearTimeout(this.timer.hide);
                    this.elem.addClass(this.options.active);
                    this.elem[0].offsetHeight; // 强制渲染
                    this.elem.addClass(this.options.transition);
                    this.timer.show = setTimeout(function() {
                        if (cb) {
                            cb();
                        }
                    }, this.options.duration);
                },
                hide: function(cb) {
                    clearTimeout(this.timer.show);
                    clearTimeout(this.timer.hide);
                    this.elem.removeClass(this.options.transition);
                    this.timer.hide = setTimeout(function() {
                        this.elem.removeClass(this.options.active);
                        if (cb) {
                            cb();
                        }
                    }.bind(this), this.options.duration);
                }
            };
            return V;
        }(),

        // 异步短信发送
        mobileMsg: {

            /**
             *  订单重发短信
             * @param  {[String]}   orderNo   订单ID
             * @param  {Function} callback 函数
             */
            orderResend: function(orderNo, callback) {
                $.ajax({
                    dataType: "json",
                    cache: false,
                    url: "/json/black/order/resendmsg/" + orderNo,
                    success: function(data) {
                        if (data.hasError === undefined || data.hasError) {
                            $GH.alert(data.message);
                        } else {
                            $GH.alert("短信发送成功！");
                            if (callback) {
                                // 成功回调
                                callback();
                            }
                        }
                    },
                    error: function(xhr, code) {
                        if (code === 'abort') {
                            return;
                        }
                        $GH.alert("发送失败，请稍后重试！");
                    }
                });
            },
        },

        /**
         *  刷新图片验证码
         * @param  {[Object]}   img   Zepto图片对象
         * @return {Object}
         */
        refreshCaptcha: function(img) {
            var src = img.attr('data-src') || img.attr('src');
            img.attr('src', src + "?_=" + Math.floor(Math.random() * 10000000));
            return img;

        },

        /**
         * 运行环境检测，检测微医APP请使用$DT.appVersion
         * @return {Object}
         */
        env: {
            isIos: function() {
                var ua = navigator.userAgent;
                return ua.match(/(iPad).*OS\s([\d_]+)/) || ua.match(/(iPod)(.*OS\s([\d_]+))?/) || ua.match(/(iPhone\sOS)\s([\d_]+)/);
            },
            isAndroid: function() {
                return navigator.userAgent.indexOf('Android') > -1;
            },
            isWeixin: function() {
                return navigator.userAgent.toLowerCase().match(/MicroMessenger/i) == "micromessenger" && (typeof wx != 'undefined');
            },
            // 微信城市服务iframe中
            isWxCityBox: function() {
                return (navigator.userAgent.toLowerCase().match(/MicroMessenger/i) == "micromessenger") && (window.top !== window.self) && (typeof wxbridge !== "undefined" && wxbridge !== null);
            },
            // 电话帮
            isDHB: function() {
                return (document.domain.indexOf("dhb") > -1) && (typeof nativeEntry != 'undefined');
            },
            // 点评
            isDianPing: function() {
                return navigator.userAgent.indexOf('com.dianping.dpscope') > -1;
            },
            // 支付宝
            isAlipay: function() {
                return navigator.userAgent.indexOf('Alipay') > -1;
            }
        },
        // 把字符串文本中的地址转换为超链接
        text2Url: function(text) {
            var re = /(http:\/\/|ftp:\/\/|https:\/\/){0,1}[\w\-_]+(\.[\w\-_]+)+([\w\-\.,@?^=%&amp;:/~\+#]*[\w\-\@?^=%&amp;/~\+#])?/ig;
            var newRe = /(http:\/\/|ftp:\/\/|https:\/\/)[\w\-_]+(\.[\w\-_]+)+([\w\-\.,@?^=%&amp;:/~\+#]*[\w\-\@?^=%&amp;/~\+#])?/ig;
            var wwwRe = new RegExp("((www.)[a-zA-Z0-9\-\.]+\.(com|cn|edu|gov|mil|net|org|biz|info|name|museum|us|ca|uk)(\:[0-9]+)*(/($|[a-zA-Z0-9\.\,\;\?\'\\\+&amp;%\$#\=~_\-]+))*)", "ig");
            text = text.replace(re, function(url) {
                if (newRe.test(url)) {
                    return "<a href='" + url + "'>" + url + "</a>";
                } else if (wwwRe.test(url)) {
                    return "<a href='http://" + url + "'>" + url + "</a>";
                } else {
                    return url;
                }
            });
            return text;
        },

        // emoji表情字符处理
        emoji: {
            encode: function(emojiStr) {
                // 将emoji表情编码防止数据库报错
                var patt = /[\ud800-\udbff][\udc00-\udfff]/g; // 检测utf16字符正则
                var str = emojiStr.replace(patt, function(char) {
                    var H, L, code;
                    if (char.length === 2) {
                        H = char.charCodeAt(0); // 取出高位
                        L = char.charCodeAt(1); // 取出低位
                        code = (H - 0xD800) * 0x400 + 0x10000 + L - 0xDC00; // 转换算法
                        return "&#" + code + ";";
                    } else {
                        return char;
                    }
                });
                return str;
            },
            decode: function(str) {
                //通过innerHTML转化一次使emoji表情的实体符号可以让浏览器解析
                var c = document.createElement('div');
                c.innerHTML = str;
                str = c.innerText || c.textContent;
                c = null;
                return str;
            }
        },
        // 设置渠道下载链接
        setDownloadLink: function(source) {
            if ($(".J_Download").length > 0) {
                if ($GH.env.isIos()) {
                    if ($GH.env.isWeixin()) {
                        $(".J_Download").attr("href", $GC.guahaoServer + "/guide/downloadGuide?iosSource=" + source);
                    } else {
                        $(".J_Download").attr("href", " https://itunes.apple.com/app/apple-store/id595277934?pt=97325052&ct=" + source + "&mt=8");
                    }
                }
            }
        },
        // 微信自定义分享（用于页面无法预先配置分享数据的情况下使用）
        weixinShare: function(wd) {
            if (!wd) {
                // 如果未自定义分享，则图片都用下面的
                wd = {
                    imgUrl: '/h5/img/weiyi-logo.png'
                };
            }

            // 直接使用wxData对象，支持实时修改分享文案
            if (!wd.title) {
                wd.title = document.title;
            }
            if (!wd.link) {
                wd.link = location.href;
            }
            if (!wd.desc) {
                wd.desc = "";
            }
            if (!wd.imgUrl) {
                wd.imgUrl = "";
            }
            if (!wd.type) {
                wd.type = "link";
            }
            if (!wd.dataUrl) {
                wd.dataUrl = "";
            }
            if (typeof wd.shareSuccess !== "function") {
                wd.shareSuccess = function() {};
            }
            if (typeof wd.shareCancel !== "function") {
                wd.shareCancel = function() {};
            }

            // 朋友圈
            window.wx.onMenuShareTimeline({
                title: wd.title,
                link: wd.link,
                imgUrl: wd.imgUrl,
                success: wd.shareSuccess,
                cancel: wd.shareCancel
            });

            // 朋友
            window.wx.onMenuShareAppMessage({
                title: wd.title,
                desc: wd.desc,
                link: wd.link,
                imgUrl: wd.imgUrl,
                type: wd.type,
                dataUrl: wd.dataUrl,
                success: wd.shareSuccess,
                cancel: wd.shareCancel
            });

            // QQ
            window.wx.onMenuShareQQ({
                title: wd.title,
                desc: wd.desc,
                link: wd.link,
                imgUrl: wd.imgUrl,
                success: wd.shareSuccess,
                cancel: wd.shareCancel
            });

            // QZone
            window.wx.onMenuShareQZone({
                title: wd.title,
                desc: wd.desc,
                link: wd.link,
                imgUrl: wd.imgUrl,
                success: wd.shareSuccess,
                cancel: wd.shareCancel
            });
        },
    };

})(Zepto);
