/**
 * PgwModal - Version 1.4
 *
 * Copyright 2014, Jonathan M. Piat
 * http://pgwjs.com - http://pagawa.com
 * 
 * Released under the GNU GPLv3 license - http://opensource.org/licenses/gpl-3.0
 */
;(function($){

    var bodyViewer;

    // 同$GH.Viewer
    var Viewer=function () {
        function V(elem, options) {
            var _options = {
                duration: 300,
                active: "active",
                transition: "transition"
            };
            this.options = $.extend({}, _options, options);
            this.timer = {
                show: -1,
                hide: -1
            };
            this.elem = elem;
        }
        V.prototype = {
            show: function() {
                clearTimeout(this.timer.show);
                clearTimeout(this.timer.hide);
                this.elem.addClass(this.options.active);
                this.timer.show = setTimeout(function() {
                    this.elem.addClass(this.options.transition);
                }.bind(this), 20);
            },
            hide: function() {
                clearTimeout(this.timer.show);
                clearTimeout(this.timer.hide);
                this.elem.removeClass(this.options.transition);
                this.timer.hide = setTimeout(function() {
                    this.elem.removeClass(this.options.active);
                }.bind(this), this.options.duration);
            }
        };
        return V;
    }();

    $.pgwModal = function(obj) {

        var pgwModal = {};	
        var defaults = {
            close: true,
            maxWidth: 500,
            //自定义样式 zhangyl1
            customClass:"",
            // 是否是简单提示信息
            popup: false,
            // 显示多久关闭
            duration: 0,
            closeOnBackgroundClick : false,
            // 确认框
            confirm: false,
            confirmButtons: [
                { name: '确定', style: 'btn', fn: function(close){  } },
                { name: '取消', style: 'btn', fn: function(close){ close(); } }
            ],
            afterClose: function(){},
            afterRender: function(){},
            xClass: "pgwmodal",
            loading: 'Loading in progress...',
            error: 'An error has occured. Please try again in a few moments.'
        };
        
        if (typeof window.pgwModalObject != 'undefined') {
            pgwModal = window.pgwModalObject;
        }

        // Merge the defaults and the user's config
        if ((typeof obj == 'object') && (! obj.pushContent)) {
            if (! obj.url && ! obj.target && ! obj.content) {
                throw new Error('PgwModal - There is no content to display, please provide a config parameter : "url", "target" or "content"');
            }

            pgwModal.config = {};
            pgwModal.config = $.extend({}, defaults, obj);
            window.pgwModalObject = pgwModal;
        }

        // Create modal container
        var create = function() {
        	var xClass = pgwModal.config.xClass;
        	
            if(pgwModal.config.popup){
                xClass = 'popup';
            }
            if(pgwModal.config.confirm){
                xClass = 'confirm';
            } 
             
            var customClass = pgwModal.config.customClass||"";
            
            var appendBody = '<div id="pgwModalWrapper" class="'+ xClass +" "+customClass+'"></div>'
                + '<div id="pgwModal" class="'+ xClass +" "+customClass+'">'
                + '<div class="pm-container">'
                + '<div class="pm-body">'
                + '<a href="javascript:void(0)" class="pm-close" onclick="$.pgwModal(\'close\')"></a>'
                + '<div class="pm-title"></div>'
                + '<div class="pm-content cntr"></div>'
                + '</div>'
                + '</div>'
                + '</div>';
            $('body').append(appendBody);
            $(document).trigger('PgwModal::Create');
            return true;
        };

        // Reset modal container
        var reset = function() {
            $('#pgwModal .pm-title, #pgwModal .pm-content').html('');

        	var xClass = pgwModal.config.xClass;
        	var customClass = pgwModal.config.customClass||"";
            
            if(pgwModal.config.popup){
                xClass = 'popup';
            }
            if(pgwModal.config.confirm){
                xClass = 'confirm';
            }
            //自定义样式
            var customClass = pgwModal.config.customClass || "";
            if(customClass){
                xClass += " "+customClass;
            }
            $('#pgwModalWrapper, #pgwModal').attr("class",xClass+' '+customClass);
            
            return true;
        };

        // Angular compilation
        var angularCompilation = function() {
            angular.element('body').injector().invoke(function($compile) {
                var scope = angular.element($('#pgwModal .pm-content')).scope();
                $compile($('#pgwModal .pm-content'))(scope);
                scope.$digest();
            });
            return true;
        };
        
        // Push content into the modal
        var pushContent = function(content) {
            $('#pgwModal .pm-content').html(content);
            if (pgwModal.config.angular) {
                angularCompilation();
            }
            // confirm 按钮植入
            if(pgwModal.config.confirm){
                var $p = $('<p>'); $p.addClass('btns');
                $.each(pgwModal.config.confirmButtons, function(i, btn){
                    var $a = $('<a>');
                    $a.attr({ "href": btn.link?btn.link:"javascript:;", "class": btn.style||"btn" });
                    $a.text(btn.name);
                    $a.on('click', function(){
                        btn.fn.call(this, _close);
                    });
                    $p.append($a);
                });
                $('#pgwModal .pm-content').append($p);
            }
            $(document).trigger('PgwModal::PushContent');
            pgwModal.config.afterRender();
            return true;
        };
        
        // Repositions the modal
        var reposition = function() {
        	 
            var windows_height = $(window).height();
            var modal_height = $('#pgwModal .pm-body')[0].offsetHeight; 
            var margin_top = Math.round((windows_height - modal_height)/3);
            if (margin_top <= 0) {
                margin_top = 10;
            }
            $('#pgwModal .pm-body').css('margin-top', margin_top);
            return true;
        };
        
        // Returns the modal data
        var getData = function() {
            return pgwModal.config.modalData;
        };
        
        // Returns the modal status
        var isOpen = function() {
            return $('body').hasClass('pgwModal');
        };

        // Close the modal
        var close = function() {
            $('#pgwModal, #pgwModalWrapper').hide();
            $('body').removeClass('pgwModal');
            reset();
            try {
                delete window.pgwModalObject; 
            } catch(e) {
                window['pgwModalObject'] = undefined; 
            }
            $(document).trigger('PgwModal::Close');
            return true;
        };

        var _close = function(){
            $('#pgwModal, #pgwModalWrapper').remove();
            $('body').removeClass('pgwModal');   // remove the body class to fix page layout issues
        }

        // Open the modal
        var open = function() {
            if ($('#pgwModal').length == 0) {
                create();
                bodyViewer = new Viewer($('#pgwModal .pm-body'));
            } else {
                reset();
                if(!bodyViewer){
                    bodyViewer=new Viewer($('#pgwModal .pm-body'));
                }
            }

            // 淡入
            bodyViewer.show();

            if (! pgwModal.config.close) {
                $('#pgwModal .pm-close').hide();
            } else {
                $('#pgwModal .pm-close').show();
            }

            if (pgwModal.config.title) {
                $('#pgwModal .pm-title').text(pgwModal.config.title);
            }

            if (pgwModal.config.maxWidth) {
                if(pgwModal.config.popup){
                    pgwModal.config.maxWidth = '70%';
                }
                if(pgwModal.config.confirm){
                    pgwModal.config.maxWidth = '85%';
                }
                $('#pgwModal .pm-body').css('max-width', pgwModal.config.maxWidth);
            }

            // Content loaded by Ajax
            if (pgwModal.config.url) {
                if (pgwModal.config.loading) {
                    $('#pgwModal .pm-content').html(pgwModal.config.loading);
                }

                var ajaxOptions = {
                    'url' : obj.url,
                    'success' : function(data) {
                        pushContent(data);
                    },
                    'error' : function() {
                        $('#pgwModal .pm-content').html(pgwModal.config.error);
                    }
                };

                if (pgwModal.config.ajaxOptions) {
                    ajaxOptions = $.extend({}, ajaxOptions, pgwModal.config.ajaxOptions);
                }

                $.ajax(ajaxOptions);
                
            // Content loaded by a html element
            } else if (pgwModal.config.target) {
                pushContent($(pgwModal.config.target).html());

            // Content loaded by a html element
            } else if (pgwModal.config.content) {
                pushContent(pgwModal.config.content);
            }

            $('#pgwModal, #pgwModalWrapper').show();
            $('body').addClass('pgwModal');
            $(document).trigger('PgwModal::Open');

            reposition();

            clearTimeout(pgwModal.timer);
            clearTimeout(pgwModal.timer+1);

            // 频繁弹框，清除timer，避免出现逻辑错误
            if(pgwModal.config.duration > 0){
                pgwModal.timer=setTimeout(function() {
                    // 淡出
                    bodyViewer.hide();
                }, pgwModal.config.duration);

                setTimeout(function(){
                    // 提示信息就直接销毁避免跟其他的冲突
                    _close();
                    if(typeof pgwModal.config.afterClose === 'function') {
                        pgwModal.config.afterClose();
                    }
                }, pgwModal.config.duration+300);
            }

            // Close on background click
            if (pgwModal.config.closeOnBackgroundClick) {
                $('#pgwModal').bind('click.PgwModalBackdrop', function(e) {
                    var targetClass = $(e.target).hasClass('pm-container');
                    var targetId = $(e.target).attr('id');
                    if (targetClass || targetId == 'pgwModal') {
                        close();
                    }
                });
            }
            
            return true;
        };

        // Choose the action
        if ((typeof obj == 'string') && (obj == 'close')) {
            return close();

        } else if ((typeof obj == 'string') && (obj == 'reposition')) {
            return reposition();

        } else if ((typeof obj == 'string') && (obj == 'getData')) {
            return getData();
            
        } else if ((typeof obj == 'string') && (obj == 'isOpen')) {
            return isOpen();

        } else if ((typeof obj == 'object') && (obj.pushContent)) {
            return pushContent(obj.pushContent);

        } else if (typeof obj == 'object') {
            return open();
        }
    }
})(window.Zepto || window.jQuery);
